#!/usr/bin/python

# 
# Copyright (c) 2017, Intel Corporation <www.intel.com>
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# - Neither the name of the Altera Corporation nor the
#   names of its contributors may be used to endorse or promote products
#   derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL ALTERA CORPORATION BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys, signal, os, time, re, subprocess, threading, getopt

# Debug options
OUTPUT  = 1;  # enable tool output
DEBUG   = 1;  # enable debug messages

# Parameters, with their default values
OPERATION 				= ""
ADDRESS 				= "0x0"
SIZE 					= ""
SOC 					= ""
CABLE 					= ""
FLASH 					= ""
BOARD                   = "devkit_revc"
CDB_ENTRY               = ""
CDB_ENTRY_PARAM         = ""
CABLE                   = ""
IMAGE_FILE              = ""
SD_BLOCK_LEN            = "0x200"
CHUNK_SIZE              = "0x8000000"
TFTP_FOLDER				= "C:/Users/rbacrau/Data/tftpboot"
TFTP_SERVER             = "192.168.10.105"
USE_TFTP                = "no"
MAC_ADDRESS             = "B4:B5:2F:A0:93:4D"
TFTP_BLOCK_SIZE			= "512"

def hex2dec(value):
	return int(value, 0)

def dec2hex(value):
	return hex(value)

def chunk_filename(chunk_no):
	return "chunk_a" + chr(ord('a') + chunk_no)

def signal_handler(signal, frame):
	# clear any active formatting
	print "\033[0m"	
	exit()

# print info 
def print_info(string):
	print "\033[1;32mINFO : " + string

# print error message and exit
def print_error(string):
	print "\033[1;31mERROR: " + string
	# clear any active formatting
	print "\033[0m"	
	exit()

# print debug information 
def print_debug(string):
	if DEBUG == 1:
		print "\033[1;36mDEBUG: " + string

# print application console output
def print_output(string):
	if OUTPUT == 1:
		sys.stdout.write("\033[1;30m" + string)		
				
# read a text file
def read_text_file(filename):
	file = open(filename)
	contents = file.read()
	file.close()
	return contents

# run a command and return its output	
def run_command(command, log_name):
	print_debug("Running " + command)
	p = subprocess.Popen(command, shell=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
	stderr_str = "";
	stdout_str = "";
	# poll stdout and write it to console
	while True:
		out = p.stdout.read(1)
		if out == '' and p.poll() != None:
			break
		if out != '':
			print_output(out)
			stdout_str = stdout_str + out				
	# poll stderr and write it to console
	while True:
		out = p.stderr.read(1)
		if out == '' and p.poll() != None:
			break
		if out != '':
			print_output(out)
			stderr_str = stderr_str + out
	
	# remove known harmless stderr messages
	stderr_str = stderr_str.replace("WARNING(CMD315): Target is not running","");
	stderr_str2 = stderr_str.replace("\n","");
	stderr_str2 = stderr_str2.replace("\r","");

	# check for stderr erorrs	
	if stderr_str2 != "" :
		print_info(stderr_str)
		
	# check for stdout status
	if re.search("was successful", stdout_str, re.M|re.I):
		return stdout_str	
	elif re.search("error", stdout_str, re.M|re.I):
		print_error(stdout_str)		
	# return the output	
	return stdout_str	
	
# Run jtagconfig to determine the JTAG connection parameters
# Note that it will always use the first detected JTAG connection
def detect_jtag():
	global CDB_ENTRY_PARAM	
	global SOC;	
	global CABLE;
	
	print_debug("Detecting JTAG Connection Parameters")
	jtagconfig_output = run_command("jtagconfig", "jtagconfig")
	if (CDB_ENTRY_PARAM == "") or (CABLE == ""):
		match = re.search("^(\d+)\) USB-BlasterII \[(.*)\]", jtagconfig_output)
		if match == None:
			print "Cannot parse the jtagconfig output: " + jtagconfig_output
			exit()
		if CABLE == "":
			CABLE = "USB-BlasterII [" + match.group(2) + "]"
			print_info("Detected JTAG cable='" + CABLE + "'")
		if CDB_ENTRY_PARAM == "":
			CDB_ENTRY_PARAM = "rvi_address=USB-BlasterII on localhost [" + match.group(2) + "]:USB-BlasterII " + match.group(2)		
			print_info("Detected cdb-entry-param='" + CDB_ENTRY_PARAM + "'")
	if SOC == "":
		if "10AS" in jtagconfig_output:
			SOC = "a10"
		elif "5CS" in jtagconfig_output:
			SOC = "cv"
		else:
			print "Cannot determine SOC from the jtagconfig output: " + jtagconfig_output
			exit()
		print_info("Detected soc='" + SOC + "'")

# detect the flash type by looking at the bsel
def detect_flash():
	global CABLE;
	global SOC;
	global FLASH;
	
	print_debug("Detecting flash type")

	QUARTUS_HPS_SCRIPT_FILE = "___tmp_quartus_hps.script";
	BSEL_ADDRESS = "";
	BSEL_REGEX = "";
	
	if (SOC == "a10"):		
		BSEL_ADDRESS = "0xFFD0600C"
		BSEL_REGEX   = "0xFFD0600C \: 0x....(.)..."
	elif (SOC == "cv"):
		BSEL_ADDRESS = "0xFFD08014"
		BSEL_REGEX   = "0xFFD08014 \: 0x.......(.)"
	else:
		print_error("Unsupported SOC: '" + SOC + "'");
		exit()

	# create script for quartus_hps
	script = open(QUARTUS_HPS_SCRIPT_FILE, 'w+')
	script.write("CPUR " + BSEL_ADDRESS + " 1\n")
	script.write("Q\n")
	script.close()

	# run quartus_hps with the script
	quartus_hps_output = run_command("cat < " + QUARTUS_HPS_SCRIPT_FILE  + " | quartus_hps --cable='" + CABLE + "' --boot=0 -o debug", "jtagconfig")		
	
	# parse the output to determine bsel
	match = re.search(BSEL_REGEX, quartus_hps_output)
	if match == None:
		print "Cannot determine BSEL from quartus_hps_output: " + quartus_hps_output;
		exit()
					
	bsel = match.group(1);
	print_debug("Detected bsel='" + bsel + "'")
	
	# determine flash from bsel
	if (bsel == "2") or (bsel == "3") or (bsel == "A") or (bsel == "B"):
		FLASH = "nand"
	elif (bsel == "6") or (bsel == "7") or (bsel == "E") or (bsel == "F"):
		FLASH = "qspi"
	elif (bsel == "4") or (bsel == "5") or (bsel == "C") or (bsel == "D"):
		FLASH = "sdmmc"
	else:
		print_error("Unsupported BSEL: '" + bsel + "'");
	print_info("Detected flash='" + FLASH + "'")
		
# Configure FPGA
def configure_fpga():
	global SOC;
	global CABLE;
	global BOARD_FOLDER;
	
	if SOC == "a10":
		SOF_FILE = BOARD_FOLDER + "/output_files/ghrd_10as066n2.sof"
	else:
		print_error("Unsupported SOC: " + SOC);

	print_info("Configuring FPGA")
	command = "quartus_pgm --no_banner --mode=jtag --cable=\"" + CABLE + "\" -o \"p;" + SOF_FILE + "\""
	output = run_command(command, "quartus_pgm")
	
# Create DS-5 script
def create_debugger_script(file_name):

	global SOC;
	global BOARD;
	global FLASH;
	global OPERATION;
	global ADDRESS;
	global SIZE;
	global BOARD_FOLDER;
	global IMAGE_FILE;
	global SD_BLOCK_LEN;
	global CHUNK_SIZE;	
	global TFTP_FOLDER;
	global TFTP_SERVER;
	global USE_TFTP;
	global MAC_ADDRESS;
	global TFTP_BLOCK_SIZE;

	print_info("Creating DS-5 script '" + file_name + "'")		

	# connect to board and reliably top the processor
	script = open(file_name, 'w+')
	script.write("stop\n")
	script.write("wait 5s\n")
	script.write("stop\n")
	script.write("wait 5s\n")
	#script.write("reset system\n")
	script.write("reset\n")
	script.write("wait 5s\n")
	script.write("stop\n")
	script.write("pause 1\n")

	SYNC_CMD  = "echo break"
	SYNC_LBL  = "do_echo"

	# run bootloader
	if SOC == "cv":

		SPL_BIN   = BOARD_FOLDER + "/software/bootloader_extcfg_semihosting/uboot-socfpga/spl/u-boot-spl"
		UBOOT_BIN = BOARD_FOLDER + "/software/bootloader_extcfg_semihosting/uboot-socfpga/u-boot"

		script.write("set semihosting enabled 1\n");
		script.write("loadfile " + SPL_BIN + " 0x0\n");
		script.write("set debug-from *$entrypoint\n");
		script.write("start\n");
		script.write("delete\n");
		script.write("tbreak spl_boot_device\n");
		script.write("cont\n");
		script.write("wait 60s\n");
		script.write("loadfile " + UBOOT_BIN + "\n");
		script.write("start\n");
		script.write("tbreak relocate_code\n");
		script.write("cont\n");
		script.write("wait 60s\n");
		script.write("symbol-file " + UBOOT_BIN + " ((gd_t*)$r8)->reloc_off\n");
	
	elif SOC == "a10":
	
		UBOOT_BIN = BOARD_FOLDER + "/software/bootloader_extcfg_semihosting/uboot-socfpga/u-boot"
		UBOOT_DTB = BOARD_FOLDER + "/software/bootloader_extcfg_semihosting/devicetree.dtb"	

		script.write("set trust-ro-sections-for-opcodes off\n")
		script.write("# Load uboot elf\n")
		script.write("loadfile " + UBOOT_BIN + "\n")
		script.write("# Set pc to entry point\n")
		script.write("set debug-from *$entrypoint\n")
		script.write("start\n")
		script.write("# Load the uboot dtb\n")
		script.write("restore " + UBOOT_DTB + " binary &_end\n")
		script.write("# Set breakpoint on panic function in case something goes wrong\n")
		script.write("b panic\n")
		script.write("echo starting u-boot\n")
		script.write("# Start uboot\n")
		script.write("thbreak " + SYNC_LBL + "\n")
		script.write("continue\n")		
		script.write("stdin " + SYNC_CMD + "\\n\n")
		script.write("wait\n")

	# start bootloader and initialize flash
	if FLASH == "nand":
		
		FLASH_CMD = "nand"		
		
		script.write("thbreak " + SYNC_LBL + "\n")
		script.write("continue\n")
		script.write("stdin nand info\\n\n")
		script.write("stdin " + SYNC_CMD + "\\n\n")
		script.write("wait\n")
		
	elif FLASH == "qspi":
		
		FLASH_CMD = "sf"		
		
		script.write("thbreak " + SYNC_LBL + "\n")
		script.write("continue\n")
		script.write("stdin sf probe\\n\n")
		script.write("stdin " + SYNC_CMD + "\\n\n")
		script.write("wait\n")

	elif FLASH == "sdmmc":
		
		FLASH_CMD = "mmc"		
		
		script.write("thbreak " + SYNC_LBL + "\n")
		script.write("continue\n")
		script.write("stdin mmc rescan\\n\n")
		script.write("stdin mmcinfo\\n\n")
		script.write("stdin " + SYNC_CMD + "\\n\n")
		script.write("wait\n")

	# deal with the input file	
	if IMAGE_FILE <> "":
		no_chunks = (os.path.getsize(IMAGE_FILE) + hex2dec(CHUNK_SIZE) - 1) / hex2dec(CHUNK_SIZE);
		run_command("rm -f chunk_*", "cmd");	
		run_command("split --bytes=" + str(hex2dec(CHUNK_SIZE)) + " " + IMAGE_FILE + " chunk_", "cmd");
		if (USE_TFTP == "yes") or (USE_TFTP == "true") or (USE_TFTP == "1"):
			USE_TFTP = "yes";
			run_command("rm -f " + TFTP_FOLDER + "/chunk_*", "cmd");		
			run_command("mv chunk_* " + TFTP_FOLDER + "/", "cmd");	

	# initialize tftp
	if USE_TFTP == "yes":
		script.write("thbreak " + SYNC_LBL + "\n")
		script.write("continue\n")
		script.write("stdin setenv ethaddr " + MAC_ADDRESS + "\\n\n")
		script.write("stdin setenv tftpblocksize " + TFTP_BLOCK_SIZE + "\\n\n")
		script.write("stdin setenv autoload no\\n\n")
		script.write("stdin dhcp\\n\n")
		script.write("stdin setenv serverip " + TFTP_SERVER + "\\n\n")
		script.write("stdin " + SYNC_CMD + "\\n\n")
		script.write("wait\n")

	# deal with the commands	
	if ("e" in OPERATION) or ("E" in OPERATION):
	
		script.write("thbreak " + SYNC_LBL + "\n")
		script.write("continue\n")			
		
		if (SIZE == "") and (ADDRESS == "0x0") and (FLASH == "nand"):
			if ("E" in OPERATION):
				script.write("stdin " + FLASH_CMD + " erase.chip clean\\n\n")
			else:
				script.write("stdin " + FLASH_CMD + " erase.chip\\n\n")
		else:
			if ("E" in OPERATION) and (FLASH == "nand"): 
				script.write("stdin " + FLASH_CMD + " erase clean " + ADDRESS + " " + SIZE + "\\n\n")
			else:
				script.write("stdin " + FLASH_CMD + " erase " + ADDRESS + " " + SIZE + "\\n\n")
			
		script.write("stdin " + SYNC_CMD + "\\n\n")
		script.write("wait\n")

	if ("p" in OPERATION) or ("P" in OPERATION):
	
		# determine currrent address 
		crt_address = hex2dec(ADDRESS);

		# determine number of chunks
		if(no_chunks == 1):
			chunk_size = hex2dec(SIZE)
		else:
			chunk_size = hex2dec(CHUNK_SIZE)

		# write all the chunks
		for c in range(0,no_chunks):

			# blank the area
			script.write("thbreak " + SYNC_LBL + "\n")
			script.write("continue\n")	
			if FLASH <> "sdmmc":
				script.write("stdin mw.b 0x00008000 0xff " + dec2hex(chunk_size) + "\\n\n")					
			script.write("stdin " + SYNC_CMD + "\\n\n")
			script.write("wait\n")		
						
			# download image
			script.write("echo loading flash image\n")
			script.write("echo download the binary image\n")

			if USE_TFTP == "yes":
				script.write("stdin tftp 0x00008000 " + chunk_filename(c) + "\\n\n")				
			else:
				script.write("restore " + chunk_filename(c) + " binary 0x00008000 \n")

			# write image
			script.write("thbreak " + SYNC_LBL + "\n")
			script.write("continue\n")
			if ("P" in OPERATION) and (FLASH == "nand"):
				script.write("stdin " + FLASH_CMD + " write.trimffs 0x00008000 " + dec2hex(crt_address) + " " + dec2hex(chunk_size) + "\\n\n")
			else:
				if FLASH == "sdmmc":
					script.write("stdin mmc write 0x00008000 " + dec2hex(crt_address/hex2dec(SD_BLOCK_LEN)) + " " + dec2hex(chunk_size/hex2dec(SD_BLOCK_LEN)) + "\\n\n")
				else:
					script.write("stdin " + FLASH_CMD + " write 0x00008000 " + dec2hex(crt_address) + " " + dec2hex(chunk_size) + "\\n\n")
			script.write("stdin " + SYNC_CMD + "\\n\n")
			script.write("wait\n")
		
			if "v" in OPERATION:
		
				# read image back and compare
				script.write("echo read back and compare the image\n")
				script.write("thbreak " + SYNC_LBL + "\n")
				script.write("continue\n")
				if FLASH == "sdmmc":
					script.write("stdin mmc read 0x08008000 " + dec2hex(crt_address/hex2dec(SD_BLOCK_LEN)) + " " + dec2hex(chunk_size/hex2dec(SD_BLOCK_LEN)) + "\\n\n")
				else:
					script.write("stdin " + FLASH_CMD + " read 0x08008000 " + dec2hex(crt_address) + " " + dec2hex(chunk_size) + "\\n\n")
				script.write("stdin cmp.b 0x00008000 0x08008000 " + dec2hex(chunk_size) + "\\n\n")				
				script.write("stdin " + SYNC_CMD + "\\n\n")
				script.write("wait\n")	

			# advance the current address
			crt_address = crt_address + chunk_size;

	# end the script
	script.write("quit\n")			
	script.close()
	
def display_log_file(filename):

	# wait for file to exist
	while True:
		if os.path.isfile(filename):
			break		
			
	# read and print file contents
	file = open(filename, 'rU')
	while 1:
		where = file.tell()
		line = file.readline()
		if not line:
			time.sleep(0.125)
			file.seek(where)
		else:
			#sys.stdout.write("\033[1;33mUBOOT: " + line)
			sys.stdout.write("\033[1;33m" + line)
		
def run_debugger(script_file_name):
	
	TARGET_CONSOLE_OUTPUT_FILE="___tmp_ds5_uboot_console.txt"
	
	print_debug("Running Arm DS debugger")	
	
	# delete previous file if it exists
	if os.path.exists(TARGET_CONSOLE_OUTPUT_FILE):
		os.remove(TARGET_CONSOLE_OUTPUT_FILE)

	# start thread to display the log file
	t = threading.Thread(target = display_log_file, args = (TARGET_CONSOLE_OUTPUT_FILE,) )
	t.daemon = True	
	t.start()
	
	# run the debugger
	command = "armdbg --continue_on_error=true --stop_on_connect=false --cdb-entry=\"" + CDB_ENTRY + "\" --cdb-entry-param=\"" + CDB_ENTRY_PARAM + "\" --semihosting-output " + TARGET_CONSOLE_OUTPUT_FILE + " -s " + script_file_name
	output = run_command(command, "armdbg")
	
	# wait for the log thread to catch up
	time.sleep(2)
	

def main():
	global OPERATION;
	global ADDRESS;
	global SIZE;
	global CABLE;
	global CDB_ENTRY_PARAM;
	global CDB_ENTRY;
	global FLASH;
	global SOC;
	global BOARD;
	global BOARD_FOLDER;
	global IMAGE_FILE;
	global SD_BLOCK_LEN;
	global CHUNK_SIZE;	
	global USE_TFTP;
	global TFTP_FOLDER;
	global TFTP_SERVER;
	global MAC_ADDRESS;
	global TFTP_BLOCK_SIZE;
	
	CONFIG_FPGA = 1;

	# register handler for CTRL-C
	signal.signal(signal.SIGINT, signal_handler)	
	
	# usage string, for help]
	USAGE = "Usage: " + sys.argv[0] + " [--board=<board_name>] [--soc=<soc_name>)] [--flash=nand|qspi|sdmmc] --operation=(combination of seEpPv) [--address=<hex_address>] [--size=<hex_size>] [--file=<file>] [--use-tftp=yes|no] [--tftp-folder=<folder>] [--tftp-server=<server>] [--mac-address=<mac>] [--help]"
	
	# parse input arguments
	try:
		opts, args = getopt.getopt(sys.argv[1:],"",["board=","soc=","flash=","operation=","address=","size=","file=","sd-block-len=","chunk-size=","use-tftp=","tftp-folder=","tftp-server=","tftp-block-size=","mac-address=","no-fpga-config","help"])
	except getopt.GetoptError:
		print USAGE;
		exit()		
	for opt, arg in opts:
		if opt in ("--board"):
			BOARD = arg		
		elif opt in ("--soc"):
			SOC = arg
		elif opt in ("--flash"):
			FLASH = arg		
		elif opt in ("--operation"):
			OPERATION = arg
		elif opt in ("--address"):
			ADDRESS = arg
		elif opt in ("--size"):
			SIZE = arg			
		elif opt in ("--file"):
			IMAGE_FILE = arg		
		elif opt in ("--sd-block-len"):
			SD_BLOCK_LEN = arg; 
		elif opt in ("--chunk-size"):
			CHUNK_SIZE = arg; 
		elif opt in ("--use-tftp"):
			USE_TFTP = arg.lower()
		elif opt in ("--tftp-folder"):
			TFTP_FOLDER = arg
		elif opt in ("--tftp-server"):
			TFTP_SERVER = arg;
		elif opt in ("--tftp-block-size"):
			TFTP_BLOCK_SIZE = arg;
		elif opt in ("--mac-address"):
			MAC_ADDRESS = arg;
		elif opt in ("--no-fpga-config"):
			CONFIG_FPGA = 0;
		elif opt in ("--help"):
			print USAGE;
			exit()

	# check that operation parameter was provided
	if OPERATION.lower() not in ("e", "p", "pv", "ep", "epv", "s"):
		print USAGE;
		exit()
	
	# check that the file was provided and exists for scenarios that need it
	if OPERATION.lower() in ("p", "pv", "ep", "epv"):
		if IMAGE_FILE == "":
			print "ERROR: A valid file name is needed when programming or verifying!"
			exit()
		if not os.path.isfile(IMAGE_FILE):
			print "ERROR: File " + IMAGE_FILE + " does not exist!"
			exit()
		# provide size if not provided
		if SIZE == "":
			SIZE = dec2hex(os.path.getsize(IMAGE_FILE))

	# disable TFTP when no file is specified
	if IMAGE_FILE == "":
		USE_TFP = "";

	# check that TFPT, if requested, has the right parameters
	if USE_TFTP == "yes":
		if TFTP_SERVER == "":
			print "ERROR: --tftp-server=<server_ip> option must be provided!"
			exit()
		if TFTP_FOLDER == "":
			print "ERROR: --tftp-fodler=<folder> option must be provided!"
			exit()
		if not  os.path.isdir(TFTP_FOLDER):
			print "ERROR: Folder " + TFTP_FOLDER + " does not exist!"
			exit()
		if not re.match(r'[0-9a-f][0-9a-f](:[0-9a-f][0-9a-f]){5}', MAC_ADDRESS, re.M | re.I):
			print "ERROR: MAC Address " + MAC_ADDRESS + " is incorrect!";
			exit();
			

	# determine DS-5 connection parameters if needed
	if (CDB_ENTRY_PARAM == "") or (SOC == "")  or (CABLE == ""):
		detect_jtag()
					
	# select cdb-entry if not provided
	if CDB_ENTRY == "":
		if SOC == "a10":
			CDB_ENTRY = "Intel SoC FPGA::Arria 10 SoC::Bare Metal Debug::Bare Metal Debug::Debug Cortex-A9_0::USB-Blaster"
		elif SOC == "cv":
			CDB_ENTRY = "Intel SoC FPGA::Cyclone V SoC (Dual Core)::Bare Metal Debug::Bare Metal Debug::Debug Cortex-A9_0::USB-Blaster"
		elif SOC == "av":
			CDB_ENTRY = "Intel SoC FPGA::Arria V SoC::Bare Metal Debug::Bare Metal Debug::Debug Cortex-A9_0::USB-Blaster"
		print_debug("Detected cdb-entry='" + CDB_ENTRY + "'")

	# determine the flash type if needed
	if FLASH == "":
		detect_flash()
	
	# determine board folder
	BOARD_FOLDER = "./" +  SOC + "_" + FLASH + "_" + BOARD + "_design"

	# check if folder exists
	if not os.path.isdir(BOARD_FOLDER):
		print_error("Cannot find board binaries folder: " + BOARD_FOLDER);	

	# configure FPGA
	if (SOC == "a10") and (CONFIG_FPGA == 1):
		configure_fpga()
	
	DEBUGGER_SCRIPT_FILE = "___tmp_ds5-debugger-script.ds5"
	create_debugger_script(DEBUGGER_SCRIPT_FILE)
	run_debugger(DEBUGGER_SCRIPT_FILE)
	
	print_debug("Finish run_debugger !!!")	

	# clear any active formatting
	print "\033[0m"	


if __name__ == "__main__":
    main()

